--
--  Platformer Tutorial
--

local loader = require "AdvTiledLoader/Loader"
-- set the path to the Tiled map files
loader.path = "maps/"

local HC = require "HardonCollider"

local camera = require("hump.camera")

local hero
local collider
local allSolidTiles
local cam
local music

function love.load()
	
	cam = camera.new()

	-- load the level and bind to variable map
	map = loader.load("bosstest1.tmx")
	map:autoDrawRange(0, 0, 1, 0)

	-- load HardonCollider, set callback to on_collide and size of 100
	collider = HC(100, on_collide)
	
	-- find all the tiles that we can collide with
	allSolidTiles = findSolidTiles(map)
	
	-- set up the hero object, set him to position 32, 32
	setupHero(32,32)
	
	music = love.audio.newSource("sound/background_music.ogg")
	music:play()
end

function love.keypressed(k)
	if k == 'escape' then
		love.event.push('quit')
	end
end

function math.sign(x)
	return x / math.abs(x)
end

function closer(x, change)
	local r = math.max(0, math.abs(x) - change)
	return r * math.sign(x)
end

function love.update(dt)
	
	-- do all the input and movement
	
	handleInput(dt)
	updateHero(dt)
	
	-- update the collision detection
	
	collider:update(dt)
	
	--cam:move(20*dt, 0)
	--cam:rotate(0.7*dt)

	--move camera
	local cam_x, cam_y = cam.pos.x, cam.pos.y
	local hero_x, hero_y = hero:center()
	local move_cam_x = closer(hero_x - cam_x, 100)
	local move_cam_y = closer(hero_y - cam_y, 100)
	cam:move(move_cam_x, move_cam_y)
end

function love.draw()
	cam:attach()

	-- scale everything 2x
	--love.graphics.scale(2,2)
		
	-- draw the level
	local cam_x = cam.pos.x - love.graphics.getWidth()/2
	local cam_y = cam.pos.y - love.graphics.getHeight()/2
	map:autoDrawRange(-cam_x, -cam_y, 1, 0)
	map:draw()
	
	-- draw the hero as a rectangle
	hero:draw("fill")
	local x, y = hero:center()
	x = x - hero.img:getWidth()/2
	y = y - hero.img:getHeight()+24
	love.graphics.draw(hero.img, x, y)

	cam:detach()
end

function on_collide(dt, shape_a, shape_b, mtv_x, mtv_y)
	
	-- seperate collision function for entities
	collideHeroWithTile(dt, shape_a, shape_b, mtv_x, mtv_y)
	collideHeroWithSpike(dt, shape_a, shape_b, mtv_x, mtv_y)
	
end

function collideHeroWithTile(dt, shape_a, shape_b, mtv_x, mtv_y)
	
	-- sort out which one our hero shape is
	local hero_shape, tileshape
	if shape_a == hero and shape_b.type == "tile" then
		hero_shape = shape_a
	elseif shape_b == hero and shape_a.type == "tile" then
		hero_shape = shape_b
	else
		-- none of the two shapes is a tile, return to upper function
		return
	end
	
	-- why not in one function call? because we will need to differentiate between the axis later
	hero_shape:move(mtv_x, 0)
	hero_shape:move(0, mtv_y)

	if math.abs(mtv_x) > 10 then hero.x_vel = 0 end
	if mtv_y ~= 0 then hero.y_vel = 0 end
end

function collideHeroWithSpike(dt, shape_a, shape_b, mtv_x, mtv_y)
	
	-- sort out which one our hero shape is
	local hero_shape, tileshape
	if shape_a == hero and shape_b.type == "spike" then
		hero_shape = shape_a
	elseif shape_b == hero and shape_a.type == "spike" then
		hero_shape = shape_b
	else
		-- none of the two shapes is a spike, return to upper function
		return
	end
	
	--die
	love.event.push('quit')
end

function setupHero(x,y)
	
	hero = collider:addRectangle(x,y,20,48)
	hero.speed = 150
	hero.accel = 600
	hero.img = love.graphics.newImage("img/character.png")

	hero.x_vel = 0
	hero.y_vel = 0

end

function handleInput(dt)
	if love.keyboard.isDown("left") then
		hero.x_vel = hero.x_vel - dt*hero.accel
		if hero.x_vel < -hero.speed then hero.x_vel = -hero.speed end
	elseif love.keyboard.isDown("right") then
		hero.x_vel = hero.x_vel + dt*hero.accel
		if hero.x_vel > hero.speed then hero.x_vel = hero.speed end
	else
		hero.x_vel = hero.x_vel - (hero.x_vel*10*dt)
		if math.abs(hero.x_vel) < 10 then hero.x_vel = 0 end
	end
	if love.keyboard.isDown("up") then
		if hero.y_vel == 0 then hero.y_vel = 300 end
	end
end

function updateHero(dt)
	
	-- apply a downward force to the hero (=gravity)
	hero.y_vel = hero.y_vel - 60*9.82*dt
	if hero.y_vel < -300 then hero.y_vel = -300 end

	hero:move(dt*hero.x_vel,0)
	hero:move(0,-dt*hero.y_vel)
end

function findSolidTiles(map)
	
	local collidable_tiles = {}
	
	for x, y, tile in map("ground"):iterate() do
		if tile.properties.Ground then
			local ctile = collider:addRectangle(x*32,y*32,32,32)
			ctile.type = "tile"
			collider:addToGroup("tiles", ctile)
			collider:setPassive(ctile)
			table.insert(collidable_tiles, ctile)
		elseif tile.properties.Spikes then
			local ctile = collider:addRectangle(x*32,y*32,32,32)
			ctile.type = "spike"
			collider:addToGroup("spikes", ctile)
			collider:setPassive(ctile)
			table.insert(collidable_tiles, ctile)
		end
	end
	
	return collidable_tiles
end
